% =======================================================
%   Demo routine for the 3 * 3 Demmel matrix 
%
%   Set doplot (line 13) TRUE if plots are wanted.
%   ----WARNING:    the plotting may take up to several minutes,
%                   depending on your computational resources.
%                   We encourage using MATLAB parallel computation
%                   facilities, since PARFOR is called in the code.
%
%   Revised:    Jun. 13, 2016.
% =======================================================

doplot = false; % Set true to plot the contour lines.
                % This may not work properly for MATLAB
                % version earlier than 2016a.

% Demmel 3 * 3 matrix
A = [   -1 -100 -10000;
         0   -1   -100;
         0    0     -1 ];
B = eye(3);
epsln = 10^-3.2;

% Solve real pseudospectral abscissa by criss-cross 
opts.alpha0 = -1;   opts.beta0=0; 
opts.tol = eps*100; opts.ctol = 1E-3; 
opts.dodisp = true;
[alpha, beta, gamma, info] = rpsa_pencil(A, B, epsln, opts); 
hstr = info.hstr;

if ~doplot
    return;
end

% Contour plots for real pesudospectrum and all the supersets
% used in the rpsa algorithm, over the grids X, Y
corder = lines(info.its); % colors for the supersets
X = [linspace(-2, -1.5, 2), linspace(-1.5, -1, 20), ...
        linspace(-1, 0.099, 20), linspace(0.1,0.399,20),...
        linspace(0.4,0.5,30)]; 
X = unique(X); % remove repeating entry
Y = [linspace(0, .2, 30), linspace(.201,.6,60), linspace(.61,1.5,60)];

%   First plot: contour of mu, and first superset, to show break down
h1=figure;
plot(-1, 0, '.k', 'Markersize', 12); hold on;   % plot EigVal
plot(-1, 0, 'ok', 'Markersize', 8, 'linewidth', 2);
contour_display(A, B, epsln, X, Y, 0, h1, []);  % draw real pseudospectrum
h3 = copyobj(h1, 'legacy');                     % make a copy of plots
contour_display(A, B, epsln, X, Y, hstr(2,3),...% draw first superset
                    h1, corder(1,:)); 
h2 = copyobj(h1, 'legacy');                     % make a copy of plots
figure(h1);                                     
plot(hstr(2,1), hstr(2,2), 'o', 'color',...     % draw h-search lines
        corder(1,:), 'Markersize', 8, 'linewidth',2);
plot(hstr(1:2,1), [0,0],'-k');                  % draw v-search lines
plot(hstr(2,1)*[1,1], hstr(3,5:6), '-xk','Markersize',15, 'linewidth', 1); 

%   Second plot: all supersets
for i = 3 : info.its 
    gamma = hstr(i,3); % ith superset
    contour_display(A, B, epsln, X, Y, gamma, h2, corder(i-1,:)); 
end
for i = 3 : info.its 
    plot(hstr(i,1), hstr(i,2), 'o', 'color', corder(i-1,:),...
            'Markersize', 8, 'linewidth',2);
end

%   Third plot: zoomin plot
figure(h3);
for i = 3 : info.its 
    plot(hstr(i,1),hstr(i,2),'o','color', corder(i,:),...
            'Markersize', 8, 'linewidth',2);
end

%   Set axis
figure(h1); xlim([-2,1]);
figure(h2); xlim([-2,1]);
plot([.1, .5, .5, .1, .1], [.2, .2, .6, .6, 0.2], '--k', 'linewidth',3);
figure(h3); axis([.1,.5,.2,.6]);

return;
