% =======================================================
%   Demo routine for the 5 * 5 Demmel matrix 
%
%   Set doplot (line 13) TRUE if plots are wanted.
%   ----WARNING:    the plotting may take up to several minutes,
%                   depending on your computational resources.
%                   We encourage using MATLAB parallel computation
%                   facilities, since PARFOR is called in the code.
%
%   Revised: Jun. 13, 2016.
% =======================================================

doplot = false; % Set true to plot the contour lines.
                % This may not work properly for MATLAB
                % version earlier than 2016a.
                
% Demmel 5 * 5 matrix
A = [   -1 -5 -25 -125 -625; 
        0  -1  -5  -25 -125; 
        0   0  -1   -5  -25; 
        0   0   0   -1   -5;
        0   0   0    0   -1]; 
B = eye(5);
epsln = .01;

% Solve real pseudospectral abscissa by criss-cross 
opts.alpha0 = -1; opts.beta0=0; opts.tol = eps*100; opts.ctol = 1E-3; opts.dodisp = true;
[alpha, beta, gamma, info] = rpsa_pencil(A, B, epsln, opts); 

if ~doplot
    return
end

hstr = info.hstr;

% Figure: contour of the first three H-search, over the grids X, Y
X = [linspace(-5,-0.4,30),linspace(-0.39,0.2,40), linspace(0.21,.5,7) ];
Y = [linspace(0,1.8,60), linspace(1.81,3,11)];

h1=figure; 

corder = [ 1, 0, 0 ;    % line colors
           0, 0.7, 0 ;
           0, 0, 1;
           0, 0, 0];

plot(-1, 0,'k.', 'Markersize', 12);             % plot EigVal
contour_display(A, B, epsln, X, Y, 0, h1, []);  % draw real pseudospectrum
contour_display(A, B, epsln, X, Y, hstr(2,3), h1, corder(1,:));  
contour_display(A, B, epsln, X, Y, hstr(3,3), h1, corder(2,:));  
contour_display(A, B, epsln, X, Y, hstr(4,3), h1, corder(3,:));  
search_display(hstr, 4, h1, corder);            % plot searching lines

% plot zoom-in region;
h2 = copyobj(h1, 'legacy'); 
axis([-.4 .2 0 1.8]);

figure(h1); % plot boundary of zoom-in region
plot([-.4 .2 .2 -.4 -.4], [0 0 1.8 1.8 0], '--k', 'linewidth', 2);

return;

