% =======================================================
%   Demo routine for the 5*5 GL matrix (a random matrix)
%
%   Set doplot (line 13) TRUE if plots are wanted.
%   ----WARNING:    the plotting may take up to several minutes,
%                   depending on your computational resources.
%                   We encourage using MATLAB parallel computation
%                   facilities, since PARFOR is called in the code.
%
%   Revised:    Jun. 13, 2016.
% =======================================================

doplot = false; % Set true to plot the contour lines.
                % This may not work properly for MATLAB
                % version earlier than 2016a.
                
% Matrix A, and perturbation level
A = [   .1019 -.8350 .2966 -.0756 -2.2079 1.2682;
        1.1813 -1.4224 -.8664 .8003 -1.3413 1.3547;
       -1.2457 -.1737 -1.1910 -.3194 -.2909 .8230;
       -.7830 -.5115 -.0109 .8860 .4878 .3264;
       -.5740 .0268 1.1950 -.1729 .9966 -.8003;
       -.3815 -.4476 -.9740 1.4030 1.0361 .7399];
B = eye(6);
epsln = 10^-.25;

% ------------------------------------------------------- %
% PART I: Solve real pseudospectral abscissa by criss-cross 
% ------------------------------------------------------- %
opts.alpha0 = 1.869378214486229E-01; % use second rightmost eigval to start
opts.beta0 = 0; 
opts.tol = eps * 100;   
opts.ctol = 1.0E-3;   
opts.dodisp = true;
opts.initck = false; % user provide initial value
[alpha, beta, gamma, info] = rpsa_pencil(A, B, epsln, opts); 

hstr = info.hstr;
e = eig(A); 

if doplot
    % Figure: contour of the first three H-search, over the grids X, Y
    X = linspace(-2, 2.5, 20);
    Y = linspace(0, 3, 60);

    %   Color to be used in the plot
    corder = [ 0, 0.6, 0 ;
               0, 0.2, 1 ;
               0, 0, 0];

    %   Figure 1: contour and first H-search
    h1 = figure; 
    plot(e,'k.', 'Markersize', 12);
    contour_display(A, B, epsln, X, Y, 0, h1);  % draw real pseudospectrum
    h2 = copyobj(h1, 'legacy');
    search_display(hstr, 1, h1, corder);            % first H-search
    title('itr = 1');
    drawnow;

    %   Figure 2: Second H-search 
    contour_display(A, B, epsln, X, Y, hstr(2,3), h2, corder(1,:)); 
    h3 = copyobj(h2, 'legacy');
    search_display(hstr, 2, h2, corder); % Second V-search
    title('itr = 2');
    drawnow;

    %   Figure 3: third V-search 
    contour_display(A, B, epsln, X, Y, hstr(3,3), h3, corder(2,:)); 
    search_display(hstr, 3, h3, corder); % first H-search
    title('itr = 3');
    drawnow;
end


% ----------------------------------------
% PART II: Monotonic reduction test
% ----------------------------------------

% case a
alpha1 = 1.5; beta1 =  1.169817951503595; 
[alpha11, gamma1, itr1, info, hstr1] = rpsa_h_search(A, B, alpha1, beta1, epsln, eps*10, true);

if doplot
    % plot the alpha-gamma curve 
    beta0 = beta1;
    X = linspace(1.5,3,60); gamma0 = linspace(0,1,101); % alpha, gamma grid
    gamma0(1) = [];
    ga = zeros(length(X),length(gamma0));
    gs = zeros(length(X),1);
    for i = 1:length(X)
        alpha0 = X(i);
        parfor j = 1:length(gamma0)
            g = gamma0(j);
            ga(i,j) = second_smallest_sv(A, B, alpha0, beta0, g);
        end
        [~,gs0] = eval_mu(A,B, alpha0, beta0); 
        gs(i) = gs0(1);
    end
    figure;
    contour(X,gamma0, log10(ga'), [log10(epsln),log10(epsln)], 'color', 'black', 'linewidth', 2);
    hold on;
    plot(X, gs,'--r','linewidth',2);
    plot(alpha11, gamma1, 'or', 'markersize', 12, 'linewidth',3);
    xlabel('\alpha');
    ylabel('\gamma', 'rotation',0);
    title(['H-searh at (', num2str(alpha1), ' , ', num2str(beta1),')']);
    drawnow;
end

% case b 
alpha2 = .5; beta2 = .4;
[alpha22, gamma2, itr2, info, hstr2] = rpsa_h_search(A, B, alpha2, beta2, epsln, eps*10, true);

if doplot
    % plot the alpha-gamma curve 
    beta0 = beta2;
    X = linspace(.5,3,60); gamma0 = linspace(0,1,101); % alpha, gamma grid
    gamma0(1) = [];
    ga = zeros(length(X),length(gamma0));
    gs = zeros(length(X),1);
    for i = 1:length(X)
        alpha0 = X(i);
        parfor j = 1:length(gamma0)
            g = gamma0(j);
            ga(i,j) = second_smallest_sv(A, B, alpha0, beta0, g);
        end
        [~,gs0] = eval_mu(A,B, alpha0, beta0); 
        gs(i) = gs0(1);
    end

    figure;
    contour(X,gamma0, log10(ga'), [log10(epsln),log10(epsln)], 'color', 'black', 'linewidth', 2);
    hold on;
    plot(X, gs,'--r','linewidth',2);
    plot(alpha22, gamma2, 'or', 'markersize', 12, 'linewidth',3);
    xlabel('\alpha');
    ylabel('\gamma', 'rotation',0);
    title(['H-searh at (', num2str(alpha2), ' , ', num2str(beta2),')']);
    drawnow
end

% END
