% =======================================================
%   Demo routine to show local optimal convergence of Rostami's
%   algorithm, for the pde problem. 
%
%   WARNING:    the simulation may take up to several hours,
%               depending on your computational resources.
%               We encourage using MATLAB parallel computation
%               facilities, since PARFOR is called in the code.
%
%   Revised:    Jun. 13, 2016.
% =======================================================

A = pde_demo(2961);
B = speye(size(A));
epsln = 0.1;

X = linspace(9.2, 10.3, 30);
Y = linspace(0, 0.8, 30);

h = figure;
[XX, YY, MM]= contour_display(A, B, epsln, X, Y, 0, h, [0 0 0]);
drawnow;

% compute eig
E = eigs(A, 20, 'LR' );
hold on;
plot(E, '.k', 'markersize', 10);
drawnow;

% run rpsas
[alpha, info] = rpsas(A, epsln);
V0 = info.V;
c= [ 1, 0, 0 ; 0, .8, 0 ; 0, 0, 1];
k = 3;  % first three iterations
for i = 1:k
    V = V0(:,1:2*i);
    contour_display(A*V, V, epsln, X, Y, 0, h, c(i,:), 1);
end
z = info.z;
plot(z(1:k+1), '-k', 'linewidth',1);
plot(z(1:k+1), '+k', 'linewidth',3,'markersize',10); 
% run Rostami's
[X,Y,U,V,zvec] = real_abscissa_initial(A, epsln, []);
plot(zvec(1:6), '-k', 'linewidth',1);
plot(zvec(1:6), 'ok', 'linewidth',3,'markersize',10); 

axis([9.2, 10.3, -.2, .8]);

save ./examples_paper/results/pde_2961.mat;



